@@ -77,11 +77,30 @@ ALawDecompressTable =[
 
       944,   912,  1008,   976,   816,   784,   880,   848]
 
-def Read_Alaw(filename):
+def Read_Alaw_1(filename):
     with open(filename, "rb") as binaryfile :
        myArr = bytearray(binaryfile.read())
     g=[ALawDecompressTable[x]/32768 for x in myArr]
     return np.array(g , dtype=np.float32)
+    
+    
+def Read_Alaw(filename,sr=8000,ffmpeg_path='c:\\ffmpeg.exe'):
+           ffmpeg_command =  '%s -ar 8000 -f alaw -i "%s" -ac 1 -ar %s -f wav pipe:1' %(ffmpeg_path ,filename,sr)
+           
+           pipe = subprocess.run(ffmpeg_command, stdout=subprocess.PIPE,   stderr=subprocess.PIPE,  bufsize=10**8)
+           
+           audio_np = np.frombuffer(buffer=pipe.stdout, dtype=np.uint16, offset=8*44)
+             
+           x =np.where(audio_np>32768)    [0]    
+           y =np.where(audio_np<=32768)    [0]  
+           
+           out_wav = np.zeros(len(audio_np) , dtype=np.float32)
+           out_wav[x] = (audio_np[x] - 32768)/32768-1
+           out_wav[y] = audio_np[y] /32768
+           
+          
+           return(out_wav) 
+       
 
 def Resample(data , fs, sr):
     """Return Resampled Data.
@@ -178,50 +197,52 @@ def Read(filename,sr=8000,ffmpeg_path='c:\\ffmpeg.exe'):   # output sampling rat
    """   
    ext = os.path.basename(filename).split('.')[-1] 
    if (ext.lower() == 'l' or ext.lower() == 'r' ):
-       if (sr==8000):
-          return( Read_Alaw(filename))
-       else:   
-          return( Resample( Read_Alaw(filename) , 8000, sr) )
-   
-   if (ext.lower() == 'mp3'):
-         return(ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path))
-   
-   File_Type_Header= np.fromfile(filename, dtype=np.byte, count=4, offset=8)
-   File_Type_Header = "".join(map(chr, File_Type_Header))   # Must be WAVE
-   if (File_Type_Header!="WAVE"):
-       return(ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path))
+         return( Read_Alaw(filename,sr=sr ,ffmpeg_path=ffmpeg_path ))
        
-   Type_of_format = np.fromfile(filename, dtype=np.int8, count=1, offset=20)[0]  # 1 is PCM   6 is alaw
+  
+   return(ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path))
+    
+   
+   
+   # if (ext.lower() == 'mp3'):
+         # return(ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path))
+   
+   # File_Type_Header= np.fromfile(filename, dtype=np.byte, count=4, offset=8)
+   # File_Type_Header = "".join(map(chr, File_Type_Header))   # Must be WAVE
+   # if (File_Type_Header!="WAVE"):
+       # return(ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path))
+       
+   # Type_of_format = np.fromfile(filename, dtype=np.int8, count=1, offset=20)[0]  # 1 is PCM   6 is alaw
 
-   ch = np.fromfile(filename, dtype=np.int8, count=1, offset=22)[0]  
+   # ch = np.fromfile(filename, dtype=np.int8, count=1, offset=22)[0]  
    
-   if (ch!=1):
-       return(ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path))
+   # if (ch!=1):
+       # return(ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path))
    
-   fs = np.fromfile(filename, dtype=np.int32, count=1, offset=24)[0] # Hz
+   # fs = np.fromfile(filename, dtype=np.int32, count=1, offset=24)[0] # Hz
    
-   if (Type_of_format!=1 and Type_of_format!=6 ):
-       return(ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path))
+   # if (Type_of_format!=1 and Type_of_format!=6 ):
+       # return(ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path))
    
-   if (Type_of_format==6):   # if file is alaw
-        byte_length = np.fromfile(filename, dtype=np.int32, count=1, offset=40)[0]
-        data = np.fromfile(filename, dtype=np.byte, count=byte_length*4, offset=44)
-        data=np.array([ALawDecompressTable[x]/32768 for x in data], dtype=np.float32)
-        return (Resample(data,fs,sr))
+   # if (Type_of_format==6):   # if file is alaw
+        # byte_length = np.fromfile(filename, dtype=np.int32, count=1, offset=40)[0]
+        # data = np.fromfile(filename, dtype=np.byte, count=byte_length*4, offset=44)
+        # data=np.array([ALawDecompressTable[x]/32768 for x in data], dtype=np.float32)
+        # return (Resample(data,fs,sr))
    
-   chunk_header = np.fromfile(filename, dtype=np.byte, count=4, offset=36)
-   chunk_header = "".join(map(chr, chunk_header))
-   if (chunk_header!="data"):
-     if (_system == 'windows'):
-      data=ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path)
-     else:
-      import librosa
-      data,_ = librosa.load(filename , sr)      
-   elif(chunk_header=="data"):
-      byte_length = np.fromfile(filename, dtype=np.int32, count=1, offset=40)[0]
-      data = np.fromfile(filename, dtype=np.int16, count=byte_length // 2, offset=44)/32768
-      data = Resample(data,fs,sr)
-   return (data)
+   # chunk_header = np.fromfile(filename, dtype=np.byte, count=4, offset=36)
+   # chunk_header = "".join(map(chr, chunk_header))
+   # if (chunk_header!="data"):
+     # if (_system == 'windows'):
+      # data=ReadMp3(filename,sr=sr, mono=True,ffmpeg_path=ffmpeg_path)
+     # else:
+      # import librosa
+      # data,_ = librosa.load(filename , sr)      
+   # elif(chunk_header=="data"):
+      # byte_length = np.fromfile(filename, dtype=np.int32, count=1, offset=40)[0]
+      # data = np.fromfile(filename, dtype=np.int16, count=byte_length // 2, offset=44)/32768
+      # data = Resample(data,fs,sr)
+   # return (data)
 
 def Write(filename, data ,fs):
 
