@@ -59,6 +59,20 @@ def _energy_activity(loge, ratio=0.4):   ##########0.9
 
 #exp(150, cost0=-5)
 
+
+def filter_sig(isig , wav , sr):
+    try:
+      w=[]
+      for i , [_,a,b,_] in enumerate(isig): 
+          w.append(wav[int(a*sr) : int(b*sr)])
+      return (np.concatenate(w)) 
+    except:
+      w=[]
+      for i , [_,a,b,_,_] in enumerate(isig): 
+          w.append(wav[int(a*sr) : int(b*sr)])
+      return (np.concatenate(w)) 
+      
+      
 def filter_output(isig , max_silence=1 ,ignore_small_speech_segments=0.5 , max_speech_len=15,split_speech_bigger_than=20):
 
    if (len(isig)==0):
